/**
 * Copyright 2020 Huawei Technologies Co., Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef MINDSPORE_LITE_INCLUDE_TRAIN_SESSION_H_
#define MINDSPORE_LITE_INCLUDE_TRAIN_SESSION_H_
#include <vector>
#include <string>
#include <tuple>
#include <unordered_map>
#include "include/lite_session.h"
#include "include/train_model.h"

namespace mindspore {
namespace session {

class TrainSession : public session::LiteSession {
 public:
  virtual ~TrainSession() = default;
  static TrainSession *CreateSession(lite::Context *context);

  virtual int CompileTrainGraph(lite::TrainModel *model) = 0;
  virtual void *ExportToBuf(char *buf, size_t *len) const = 0;
  virtual void Train() = 0;
  bool IsTrain() { return train_mode_ == true; }
  virtual void Eval() = 0;
  bool IsEval() { return train_mode_ == false; }

 protected:
  bool train_mode_ = false;
};
}  // namespace session
}  // namespace mindspore
#endif  // MINDSPORE_LITE_INCLUDE_TRAIN_SESSION_H_
